"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; } function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }// src/index.ts
var _path = require('path');
var _paramcase = require('param-case');

// src/utils.ts
var _module = require('module'); var _module2 = _interopRequireDefault(_module);

var _fs = require('fs');
var _core = require('@swc/core');
var createRequire = _module2.default.createRequire || _module2.default.createRequireFromPath;
var parseImportModule = function(code, libList) {
  const ast = _core.parseSync.call(void 0, code, {
    syntax: "ecmascript",
    jsx: true
  }).body;
  const importMaps = {};
  ast.forEach((astNode) => {
    if (astNode.type !== "ImportDeclaration") return;
    const libraryName = astNode.source.value;
    const matchLib = libList.find((lib) => lib.libraryName === libraryName);
    if (!matchLib) return;
    if (astNode.specifiers.length === 0) return;
    astNode.specifiers.forEach((item) => {
      if (item.type === "ImportNamespaceSpecifier") {
        console.warn(`Can't transform code`);
        return;
      }
      let name;
      if (item.type === "ImportDefaultSpecifier") {
        name = "default";
      } else if (item.local.type === "Identifier") {
        name = item.local.value;
      } else if (item.local.name) {
        name = item.local.name;
      }
      if (importMaps[libraryName]) {
        importMaps[libraryName].push(name);
      } else {
        importMaps[libraryName] = [name];
      }
    });
  });
  return { importMaps };
};
var codeIncludesLibraryName = (code, libList = []) => {
  return !libList.every(({ libraryName }) => {
    return !new RegExp(`('${libraryName}')|("${libraryName}")`).test(code);
  });
};
var stylePathHandler = (stylePath, ignoreStylePathNotFound = true) => {
  let str = "";
  if (isString(stylePath) && stylePath) {
    str += stylePathNotFoundHandler(stylePath, ignoreStylePathNotFound);
  } else if (Array.isArray(stylePath)) {
    stylePath.forEach((item) => {
      str += stylePathNotFoundHandler(item, ignoreStylePathNotFound);
    });
  }
  return str;
};
var stylePathNotFoundHandler = (stylePath, ignoreStylePathNotFound) => {
  if (ignoreStylePathNotFound) {
    let stylePathExists = true;
    try {
      createRequire(process.cwd() + _path.sep).resolve(stylePath);
    } catch (error) {
      stylePathExists = _optionalChain([error, 'optionalAccess', _ => _.code]) !== "MODULE_NOT_FOUND";
    }
    if (!stylePathExists) {
      const fullStylePath = _path.resolve.call(void 0, process.cwd(), "node_modules", stylePath);
      const lastPath = fullStylePath.split("/").pop();
      if (!_optionalChain([lastPath, 'optionalAccess', _2 => _2.includes, 'call', _3 => _3(".")])) {
        const possibleEndWithsPaths = [
          "/index.js",
          "/index.mjs",
          ".js",
          ".mjs"
        ];
        if (possibleEndWithsPaths.some((p) => _fs.existsSync.call(void 0, fullStylePath + p))) {
          stylePathExists = true;
        }
      }
    }
    if (stylePathExists) {
      return `import '${stylePath}';`;
    } else {
      console.warn(`${stylePath} is not found!`);
      return "";
    }
  }
  return `import '${stylePath}';`;
};
var isString = (obj) => typeof obj === "string";
var isFunction = (x) => Object.prototype.toString.call(x) == "[object Function]";
var isBool = (obj) => typeof obj === "boolean";

// src/index.ts
var swcImportLoader = function(content) {
  const options = this.getOptions();
  if (options.rspackExperiments && options.rspackExperiments.import) {
    const libList = options.rspackExperiments.import || [];
    const ignoreStylePathNotFound = options.rspackExperiments.ignoreStylePathNotFound || true;
    const hasCodeIncludesLibraryName = codeIncludesLibraryName(content, libList);
    if (hasCodeIncludesLibraryName) {
      const { importMaps } = parseImportModule(content, libList);
      let importStr = "";
      libList.forEach(({ libraryName, libraryDirectory, style, camel2DashComponentName = true }) => {
        if (importMaps[libraryName]) {
          importMaps[libraryName].forEach((item) => {
            if (camel2DashComponentName) {
              item = _paramcase.paramCase.call(void 0, item);
            }
            let stylePath;
            let prefixPath = libraryName;
            if (libraryDirectory) {
              prefixPath = _path.join.call(void 0, libraryName, libraryDirectory);
            }
            if (isFunction(style)) {
              stylePath = style(_path.join.call(void 0, prefixPath, item), item);
            } else if (isBool(style)) {
              stylePath = _path.join.call(void 0, prefixPath, item, "style");
            } else if (style == "css") {
              stylePath = _path.join.call(void 0, prefixPath, item, "style", "css");
            } else {
              stylePath = _path.join.call(void 0, prefixPath, item);
            }
            const styleImportString = stylePathHandler(stylePath, ignoreStylePathNotFound);
            if (styleImportString) {
              importStr += styleImportString + "\n";
            }
          });
        }
      });
      return importStr + content;
    }
  }
  return content;
};
var index_default = swcImportLoader;


exports.default = index_default;
